package eu.mhsl.craftattack.spawn.api.client;

import com.google.gson.Gson;
import org.apache.http.client.utils.URIBuilder;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.function.Consumer;

public class HttpRepository extends Repository {
    Consumer<URIBuilder> baseUriBuilder;
    public HttpRepository(URI basePath, Consumer<URIBuilder> baseUriBuilder) {
        super(basePath);
        this.baseUriBuilder = baseUriBuilder;
    }

    public record RequestResponse<TData>(int status, TData data) {
    }

    protected <TInput, TOutput> RequestResponse<TOutput> post(String command, TInput data, Class<TOutput> clazz) {
        return this.post(command, uriBuilder -> baseUriBuilder.accept(uriBuilder), data, clazz);
    }
    protected <TInput, TOutput> RequestResponse<TOutput> post(String command, Consumer<URIBuilder> parameters, TInput data, Class<TOutput> clazz) {
        HttpRequest request = this.getRequestBuilder(this.getUri(command))
            .POST(HttpRequest.BodyPublishers.ofString(new Gson().toJson(data)))
            .build();

        RequestResponse<String> rawResponse = this.runRequest(request);
        return new RequestResponse<>(rawResponse.status, new Gson().fromJson(rawResponse.data, clazz));
    }

    protected <TOutput> RequestResponse<TOutput> get(String command, Class<TOutput> clazz) {
        HttpRequest request = this.getRequestBuilder(this.getUri(command))
            .GET()
            .build();

        RequestResponse<String> rawResponse = this.runRequest(request);
        return new RequestResponse<>(rawResponse.status, new Gson().fromJson(rawResponse.data, clazz));
    }

    private URI getUri(String command) {
        try {
            URIBuilder builder = new URIBuilder(this.basePath + "/" + command);
            this.baseUriBuilder.accept(builder);
            return builder.build();
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }

    private HttpRequest.Builder getRequestBuilder(URI endpoint) {
        return HttpRequest.newBuilder()
            .uri(endpoint)
            .header("Content-Type", "application/json");
    }

    private RequestResponse<String> runRequest(HttpRequest request) {
        try(HttpClient client = HttpClient.newHttpClient()) {
            HttpResponse<String> httpResponse = client.send(request, HttpResponse.BodyHandlers.ofString());

            return new RequestResponse<>(httpResponse.statusCode(), httpResponse.body());
        } catch(IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
    }
}
