package eu.mhsl.craftattack.spawn.util.api;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.appliances.whitelist.Whitelist;
import eu.mhsl.craftattack.spawn.config.Configuration;
import eu.mhsl.craftattack.spawn.util.text.DisconnectInfo;
import org.apache.http.client.utils.URIBuilder;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.util.Objects;

public class ApiUtil {
    private static URI whitelistEndpoint;
    private static URI reportEndpoint;
    private static String baseUrl;
    private static String apiSecret;

    public static String getApiSecret() {
        if(apiSecret == null) {
            apiSecret = Objects.requireNonNull(Configuration.cfg.getConfigurationSection("api")).getString("secret");
        }
        return apiSecret;
    }

    public static String getBaseUrl() {
        if(baseUrl == null) {
            baseUrl = Objects.requireNonNull(Configuration.cfg.getConfigurationSection("api")).getString("baseurl");
        }
        return baseUrl;
    }

    public static URI getWhitelistEndpoint() {
        if(whitelistEndpoint == null) {
            String configValue = Objects.requireNonNull(Configuration.cfg.getConfigurationSection("api")).getString("whitelist");
            whitelistEndpoint = URI.create(getBaseUrl() + Objects.requireNonNull(configValue));
        }
        return whitelistEndpoint;
    }

    public static URI getReportEndpoint() {
        if(reportEndpoint == null) {
            String configValue = Objects.requireNonNull(Configuration.cfg.getConfigurationSection("api")).getString("report");
            reportEndpoint = URI.create(getBaseUrl() + Objects.requireNonNull(configValue));
        }
        return reportEndpoint;
    }



    public static HttpResponse<String> getHttpResponse(URI endpoint, ) {
        URIBuilder uriBuilder = new URIBuilder(endpoint);
        uriBuilder.addParameter("secret", apiSecret);
        uriBuilder.addParameter("uuid", uuid.toString());

        try(HttpClient client = HttpClient.newHttpClient()) {
            HttpRequest httpRequest = HttpRequest.newBuilder()
                .uri(uriBuilder.build())
                .header("Content-Type", "application/json")
                .GET()
                .build();

            HttpResponse<String> httpResponse = client.send(httpRequest, HttpResponse.BodyHandlers.ofString());

            if(httpResponse.statusCode() == 404)
                throw new DisconnectInfo.Throwable(
                    "Nicht angemeldet",
                    "Du bist derzeit nicht als Teilnehmer des CraftAttack-Projektes registriert!",
                    "Melde Dich bei einem Admin für eine nachträgliche Anmeldung.",
                    uuid
                );

            return new Gson().fromJson(httpResponse.body(), Whitelist.UserData.class);

        } catch(IOException | InterruptedException | URISyntaxException e) {
            throw new RuntimeException(e);
        }
    }
}
